@@ -191,9 +191,6 @@ def fromul4(template, variables="variables", indent=0):
 	def make_literal(content):
 		result.append(specials.literal(content))
 
-	def make_expression(content):
-		result.append(expression(content))
-
 	def make_scriptlet(content):
 		line = "%s%s\n" % ("\t"*indent, content)
 		if result and isinstance(result[-1], scriptlet):
@@ -258,10 +255,10 @@ def fromul4(template, variables="variables", indent=0):
 		elif opcode.code == "addvar":
 			name = _string(opcode.arg)
 			make_scriptlet("%s.put(%s, com.livinglogic.ul4.Utils.add(%s.get(%s), r%d));" % (variables, name, variables, name, opcode.r1))
-		elif opcode.code == "addvar":
+		elif opcode.code == "subvar":
 			name = _string(opcode.arg)
 			make_scriptlet("%s.put(%s, com.livinglogic.ul4.Utils.sub(%s.get(%s), r%d));" % (variables, name, variables, name, opcode.r1))
-		elif opcode.code == "addvar":
+		elif opcode.code == "mulvar":
 			name = _string(opcode.arg)
 			make_scriptlet("%s.put(%s, com.livinglogic.ul4.Utils.mul(%s.get(%s), r%d));" % (variables, name, variables, name, opcode.r1))
 		elif opcode.code == "truedivvar":
@@ -286,9 +283,9 @@ def fromul4(template, variables="variables", indent=0):
 		elif opcode.code == "getslice2":
 			make_scriptlet("r%d = com.livinglogic.ul4.Utils.getSlice(r%d, null, r%d);" % (opcode.r1, opcode.r2, opcode.r3))
 		elif opcode.code == "print":
-			make_expression("org.apache.commons.lang.ObjectUtils.toString(r%d)" % opcode.r1)
+			make_scriptlet("out.write(org.apache.commons.lang.ObjectUtils.toString(r%d));" % opcode.r1)
 		elif opcode.code == "printx":
-			make_expression("com.livinglogic.ul4.Utils.xmlescape(org.apache.commons.lang.ObjectUtils.toString(r%d))" % opcode.r1)
+			make_scriptlet("out.write(com.livinglogic.ul4.Utils.xmlescape(org.apache.commons.lang.ObjectUtils.toString(r%d)));" % opcode.r1)
 		elif opcode.code == "for":
 			loopcounter += 1
 			make_scriptlet("for (java.util.Iterator iterator%d = com.livinglogic.ul4.Utils.iterator(r%d); iterator%d.hasNext();)" % (loopcounter, opcode.r2, loopcounter))
@@ -519,6 +516,11 @@ def fromul4(template, variables="variables", indent=0):
 				make_scriptlet("r%d = com.livinglogic.ul4.Utils.find(r%d, r%d, r%d, r%d);" % (opcode.r1, opcode.r2, opcode.r3, opcode.r4, opcode.r5))
 			else:
 				raise ul4c.UnknownMethodError(opcode.arg)
+		elif opcode.code == "callmethkw":
+			if opcode.arg == "render":
+				make_scriptlet("r%d = ((com.livinglogic.ul4.Template)r%d).renders((java.util.Map)r%d);" % (opcode.r1, opcode.r2, opcode.r3))
+			else:
+				raise ul4c.UnknownMethodError(opcode.arg)
 		elif opcode.code == "if":
 			make_scriptlet("if (com.livinglogic.ul4.Utils.getBool(r%d))" % opcode.r1)
 			make_scriptlet("{")
@@ -533,7 +535,7 @@ def fromul4(template, variables="variables", indent=0):
 			indent -= 1
 			make_scriptlet("}")
 		elif opcode.code == "render":
-			make_scriptlet("((com.livinglogic.ul4.JSPTemplate)r%d).execute(out, (Map)r%d);" % (opcode.r1, opcode.r2))
+			make_scriptlet("((com.livinglogic.ul4.Template)r%d).renderjsp(out, (Map)r%d);" % (opcode.r1, opcode.r2))
 		else:
 			raise ul4c.UnknownOpcodeError(opcode.code)
 	make_scriptlet("//@@@ END template code")
